@@ -322,20 +322,32 @@ def _group_insert_data(data,group_size=500,semicolon=True):
     max_groups = int(total_rows / group_size)
     print(f"    Total rows: {total_rows}")
     print(f"    max_groups: {max_groups}")
+    chunks = _lu.chunk_list(data,group_size)
 
-
-    for idx in range(max_groups):
-        start_idx = group_size * idx
-        end_idx = start_idx + group_size
-        if end_idx > len(data):
-            end_idx = len(data)
-        raw_group = data[start_idx:end_idx]
-
+    # chunks = [data[x:x+group_size] for x in range(0, len(data), group_size)]
+    for chunk in chunks:
         insert_group = []
-        for dd in raw_group:
+        for dd in chunk:
             dd = _csu.unicode_to_html(dd,semicolon=semicolon)
             insert_group.append(dd)
         inserts.append(insert_group)
+    # for idx in range(max_groups):
+    #     start_idx = group_size * idx
+    #     end_idx = start_idx + group_size
+
+    #     if end_idx > len(data):
+    #         end_idx = len(data)
+    #     print(f"{start_idx}:{end_idx}")
+    #     raw_group = data[start_idx:end_idx]
+
+    #     insert_group = []
+    #     for dd in raw_group:
+    #         dd = _csu.unicode_to_html(dd,semicolon=semicolon)
+    #         insert_group.append(dd)
+    #     inserts.append(insert_group)
+
+
+
         # sql = c.string.sql.insert_sql(insert_group,"cities","idealech_Equari_Meta")
         # print(f"sql: {sql}")
 
@@ -411,6 +423,7 @@ def insert_sql(data,table_name,schema_name=None,**kwargs):
         row_count = len(data)
         if row_count > 0:
             if row_count > group_size:
+                print(f"Grouping inserting data for {table_name}.")
                 # @Mstep [] slice the data into smaller groups.
                 inserts = _group_insert_data(data,group_size)
                 output_sql = ''
@@ -436,7 +449,7 @@ def insert_sql(data,table_name,schema_name=None,**kwargs):
                 value_array.append(_gen_value_string(row,columns))
             # @Mstep [] generate the column string.
             column_string = _gen_column_string(columns)
-            
+
             # @Mstep [] join the value_array with commas
             value_string = ',\n'.join(value_array)
             # @Mstep [] replace the literal string "__NULL_VALUE__" with null
@@ -458,6 +471,57 @@ def insert_sql(data,table_name,schema_name=None,**kwargs):
             # cfu.write.write(path,sql)
             return sql
 #     return False
+
+def insert_from_dict(data:dict,table_name,schema_name=None):
+    table = f"`{schema_name}`.`{table_name}`"
+    if schema_name is None:
+        table = f"`{table_name}`"
+
+    column_string = _gen_column_string(data.keys())
+
+    value_string = ','.join(['%s' for x in list(range(len(data.keys())))])
+    # for k,v in data.items():
+    sql = f"INSERT INTO {table} ({column_string}) VALUES ({value_string})"
+    return (sql,list(data.values()))
+
+
+def select_from_dict(data:dict,table_name,**kwargs):
+    schema_name = _obj.get_kwarg(['schema_name'],None,(str),**kwargs)
+    select = _obj.get_kwarg(['select'],["*"],(list,dict),**kwargs)
+    if isinstance(select,(dict)):
+        select = _obj.keys_to_list(select)
+    
+    select_string = ', '.join(select)
+    
+    table = f"`{schema_name}`.`{table_name}`"
+    if schema_name is None:
+        table = f"`{table_name}`"
+
+    output_data = {}
+    wheres = []
+    for k,v in data.items():
+        if v is None:
+            wheres.append(f"{k} is NULL")
+            continue
+        if _csu.to_snake_case(v) == "not_null":
+            wheres.append(f"{k} is not NULL")
+            continue
+            
+        if k.lower() not in ['limit','offset']:
+            wheres.append(f"{k}=${k}")
+            output_data[k] = v
+        
+
+
+    sql = F"SELECT {select_string} FROM {table}"
+
+
+    if len(wheres)> 0:
+        wheres_string = ' AND '.join(wheres)
+        sql = f"{sql} WHERE {wheres_string}"
+    return sql,output_data
+
+
 # def insert_sql(data,table_name,schema_name=None):
 #     '''
 #         Generate an insert SQL statement from a list of data.
@@ -534,7 +598,7 @@ def insert_sql(data,table_name,schema_name=None,**kwargs):
 #                 value_array.append(_gen_value_string(row,columns))
 #             # @Mstep [] generate the column string.
 #             column_string = _gen_column_string(columns)
-            
+
 #             # @Mstep [] join the value_array with commas
 #             value_string = ',\n'.join(value_array)
 #             # @Mstep [] replace the literal string "__NULL_VALUE__" with null
@@ -744,9 +808,12 @@ def escape_quoted_chars(value,reverse=False,ignore=None):
         `value` {str}
             The string to search within
 
-        `reverse` {bool}
+        [`reverse`=False] {bool}
             if True it will reverse the escaped chars with their actual chars.
 
+        [`ignore`=None] {list,str}
+            The characters to IGNORE.
+
 
         Return {str}
         ----------------------
@@ -801,7 +868,7 @@ def parse_column_data(value=None):
     match = _re.findall(r"\s+DEFAULT\s+([\'\"][^\']*[\'\"]|[0-9\.])",value,_re.IGNORECASE)
     if match is not None:
         print(f"default found: {match}")
-    
+
     # value = "`timestamp`       int NULL DEFAULT NULL COMMENT 'Timestamp of when the task was created.' ,"
     _tick,_double_quote,_single_quote,_comma = [Suppress(x) for x in list('`"\',')]
     _quote = (_tick|_double_quote|_single_quote)
@@ -886,7 +953,7 @@ def parse_column_data(value=None):
     return output
 
 def parse_schema_statement(value=None):
-    
+
     # value = "CREATE SCHEMA IF NOT EXISTS `idealech_Equari_Content_Database`;"
 
 
@@ -931,7 +998,7 @@ def parse_schema_statement(value=None):
     return output
 
 def parse_table_statement(value=None):
-    
+
     # value = "DROP TABLE IF EXISTS `idealech_Equari_Management_Database`.`blackholes`;"
 
 
